@@ -505,6 +505,7 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 
 	case 206: /* write_queue is already prepared */
 		break;
+	case 204:
 	case 205: /* class: header only */
 	case 304:
 	default:
@@ -524,39 +525,51 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 		    (con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) == 0) {
 			off_t qlen = chunkqueue_length(con->write_queue);
 
-			/* if we have no content for a GET/PORT request, send Content-Length: 0
-			 * if it is a HEAD request, don't generate a Content-Length as 
-			 * the backend might have already cut it off */
-			if (qlen > 0 || con->request.http_method != HTTP_METHOD_HEAD) {
+			/**
+			 * The Content-Length header only can be sent if we have content:
+			 * - HEAD doesn't have a content-body (but have a content-length)
+			 * - 1xx, 204 and 304 don't have a content-body (RFC 2616 Section 4.3)
+			 *
+			 * Otherwise generate a Content-Length header as chunked encoding is not 
+			 * available
+			 */
+			if ((con->http_status >= 100 && con->http_status < 200) ||
+			    con->http_status == 204 ||
+			    con->http_status == 304) {
+				/* no Content-Body, no Content-Length */
+			} else if (qlen > 0) {
 				buffer_copy_off_t(srv->tmp_buf, chunkqueue_length(con->write_queue));
 
 				response_header_overwrite(srv, con, CONST_STR_LEN("Content-Length"), CONST_BUF_LEN(srv->tmp_buf));
 			}
 		}
 	} else {
-		/* disable keep-alive if size-info for the body is missing */
-		if ((con->parsed_response & HTTP_CONTENT_LENGTH) &&
+		/**
+		 * the file isn't finished yet, but we have all headers
+		 *
+		 * to get keep-alive we either need:
+		 * - Content-Length: ... (HTTP/1.0 and HTTP/1.0) or
+		 * - Transfer-Encoding: chunked (HTTP/1.1)
+		 */
+
+		if (((con->parsed_response & HTTP_CONTENT_LENGTH) == 0) &&
 		    ((con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) == 0)) {
 			con->keep_alive = 0;
 		}
 
-		if (0 == (con->parsed_response & HTTP_CONNECTION)) {
-			/* (f)cgi did'nt send Connection: header
-			 *
-			 * shall we ?
-			 */
-			if (((con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) == 0) &&
-			    (con->parsed_response & HTTP_CONTENT_LENGTH) == 0) {
-				/* without content_length, no keep-alive */
-
-				con->keep_alive = 0;
-			}
-		} else {
+		/**
+		 * if the backend sent a Connection: close, follow the wish
+		 *
+		 * NOTE: if the backend sent Connection: Keep-Alive, but no Content-Length, we
+		 * will close the connection. That's fine. We can always decide the close 
+		 * the connection
+		 *
+		 * FIXME: to be nice we should remove the Connection: ... 
+		 */
+		if (con->parsed_response & HTTP_CONNECTION) {
 			/* a subrequest disable keep-alive although the client wanted it */
 			if (con->keep_alive && !con->response.keep_alive) {
 				con->keep_alive = 0;
-
-				/* FIXME: we have to drop the Connection: Header from the subrequest */
 			}
 		}
 	}
@@ -1412,42 +1425,42 @@ int connection_state_machine(server *srv, connection *con) {
 
 			switch (r = http_response_prepare(srv, con)) {
 			case HANDLER_FINISHED:
-				if (con->http_status == 404 ||
-				    con->http_status == 403) {
-					/* 404 error-handler */
+				if (con->mode == DIRECT) {
+					if (con->http_status == 404 ||
+					    con->http_status == 403) {
+						/* 404 error-handler */
 
-					if (con->in_error_handler == 0 &&
-					    (!buffer_is_empty(con->conf.error_handler) ||
-					     !buffer_is_empty(con->error_handler))) {
-						/* call error-handler */
+						if (con->in_error_handler == 0 &&
+						    (!buffer_is_empty(con->conf.error_handler) ||
+						     !buffer_is_empty(con->error_handler))) {
+							/* call error-handler */
 
-						con->error_handler_saved_status = con->http_status;
-						con->http_status = 0;
+							con->error_handler_saved_status = con->http_status;
+							con->http_status = 0;
 
-						if (buffer_is_empty(con->error_handler)) {
-							buffer_copy_string_buffer(con->request.uri, con->conf.error_handler);
-						} else {
-							buffer_copy_string_buffer(con->request.uri, con->error_handler);
+							if (buffer_is_empty(con->error_handler)) {
+								buffer_copy_string_buffer(con->request.uri, con->conf.error_handler);
+							} else {
+								buffer_copy_string_buffer(con->request.uri, con->error_handler);
+							}
+							buffer_reset(con->physical.path);
+
+							con->in_error_handler = 1;
+
+							connection_set_state(srv, con, CON_STATE_HANDLE_REQUEST);
+
+							done = -1;
+							break;
+						} else if (con->in_error_handler) {
+							/* error-handler is a 404 */
+
+							con->http_status = con->error_handler_saved_status;
 						}
-						buffer_reset(con->physical.path);
-
-						con->in_error_handler = 1;
-
-						connection_set_state(srv, con, CON_STATE_HANDLE_REQUEST);
-
-						done = -1;
-						break;
 					} else if (con->in_error_handler) {
-						/* error-handler is a 404 */
-
-						con->http_status = con->error_handler_saved_status;
+						/* error-handler is back and has generated content */
+						/* if Status: was set, take it otherwise use 200 */
 					}
-				} else if (con->in_error_handler) {
-					/* error-handler is back and has generated content */
-					/* if Status: was set, take it otherwise use 200 */
-					con->http_status = con->error_handler_saved_status;
 				}
-
 				if (con->http_status == 0) con->http_status = 200;
 
 				/* we have something to send, go on */
